# -*- coding: utf-8 -*-
# author:shihua
# coder:shihua
# 这是一个具体数据模型类整理模块
"""
模块介绍
-------

这是一个具体数据模型类整理模块

设计模式：

    （1）  无 

关键点：    

    （1）fiche具体数据模型类整理

主要功能：            

    （1）fiche具体数据模型类整理                                

使用示例
-------


类说明
------

"""



####### 载入程序包 ##########################################################
############################################################################



from fiche.orm import *
from fiche.util import *
import yaml
import fiche as fc



####### fiche具体数据模型类整理 ##############################################
### 设计模式：                                                            ###
### （1）无                                                              ###
### 关键点：                                                             ###
### （1）fiche具体数据模型类整理                                          ###
### 主要功能：                                                           ###
### （1）fiche具体数据模型类整理                                          ###
############################################################################



####### fiche 元数据配置信息管理 #########################################################################
########################################################################################################



### 锁定包路径---->>进入路径开启yaml配置文件---->>解析yaml文件
fiche_package_path = fc.__file__.replace('__init__.py','')
fiche_config_file = open('{}fiche_config.yaml'.format(fiche_package_path),encoding='UTF-8')
fiche_config_yaml = yaml.load(fiche_config_file,Loader=yaml.FullLoader)



####### fiche具体数据模型类 #############################################################################
########################################################################################################



### 观察者信息
class Observer_Info(Models):
    '''
    类介绍：

        这是一个观察者信息的数据模型

    属性功能：
        name (str): 观察者名称
        state (str): 观察者状态
        summary (str): 简述
        config (str): 配置信息
        remark (str): 备注
    ''' 


    name = TagsField(name='name',tag_key=True)
    version = TagsField(name='version',tag_key=False)
    summary = TagsField(name='summary',tag_key=False)
    config = TagsField(name='config',tag_key=False)
    remark = TagsField(name='remark',tag_key=False)



###  算法包信息
class Algorithm_Info(Models):
    '''
    类介绍：

        这是一个算法包信息的数据模型

    属性功能：
        name (str): 算法包名称
        version (str): 版本
        summary (str): 简述
        config (str): 配置指导
        ramark (str): 备注
        homepage (str): 官方文档
        author (str): 作者
        authoremail (str): 作者邮箱
        license (str): 开源协议
        requires (str): 依赖
        requiredby (str): 被依赖
    '''


    name = TagsField(name='name',tag_key=True)
    version = TagsField(name='version',tag_key=True)
    summary = TagsField(name='summary',tag_key=False)
    config = TagsField(name='config',tag_key=False)
    remark = TagsField(name='remark',tag_key=False)
    homepage = TagsField(name='homepage',tag_key=False)
    author = TagsField(name='author',tag_key=False)
    authoremail = TagsField(name='authoremail',tag_key=False)
    license = TagsField(name='license',tag_key=False)
    requires = TagsField(name='requires',tag_key=False)
    requiredby = TagsField(name='requiredby',tag_key=False)



### 模型信息
class Model_Info(Models):
    '''
    类介绍：

        这是一个模型信息的数据模型

    属性功能：
        name (str): 算法包名称
        version (str): 版本
        summary (str): 简述
        data (str): 依赖数据
        algorithm (str): 依赖算法
        requires (str): 依赖
        config (str): 配置
        remark (str): 备注
    '''


    name = TagsField(name='name',tag_key=True)
    version = TagsField(name='version',tag_key=True)
    summary = TagsField(name='summary',tag_key=False)
    config = TagsField(name='config',tag_key=False)
    remark = TagsField(name='remark',tag_key=False)
    requires = TagsField(name='requires',tag_key=False)
    data = TagsField(name='data',tag_key=False)
    algorithm = TagsField(name='algorithm',tag_key=False)



### 参数信息
class Parameter_Info(Models):
    '''
    类介绍：

        这是一个参数信息的数据模型

    属性功能：
        name (str): 算法包名称
        version (str): 版本
        summary (str): 简述
        config (str): 配置指导
        remark (str): 备注
        datatype (str): 类型
        requiredby (str): 被依赖
    '''


    name = TagsField(name='name',tag_key=True)
    version = TagsField(name='version',tag_key=True)
    summary = TagsField(name='summary',tag_key=False)
    config = TagsField(name='config',tag_key=False)
    remark = TagsField(name='remark',tag_key=False)
    datatype = TagsField(name='datatype',tag_key=False)
    requiredby = TagsField(name='requiredby',tag_key=False)



### 应用信息
class Application_Info(Models):
    '''
    类介绍：

        这是一个应用信息的数据模型

    属性功能：
        name (str): 算法包名称
        version (str): 版本
        summary (str): 简述
        config (str): 配置指导
        remark (str): 备注
        requires (str): 依赖
        project (str): 项目        
    '''


    name = TagsField(name='name',tag_key=True)
    version = TagsField(name='version',tag_key=True)
    summary = TagsField(name='summary',tag_key=False)
    config = TagsField(name='config',tag_key=False)
    remark = TagsField(name='remark',tag_key=False)
    requires = TagsField(name='requires',tag_key=False)
    project = TagsField(name='project',tag_key=False)



### 数据集信息
class Dataset_Info(Models):
    '''
    类介绍；

        这是一个数据集信息的数据模型

    属性功能：
        name (str): 算法包名称
        version (str): 版本
        summary (str): 简述
        config (str): 配置指导
        remark (str): 备注
        requiredby (str): 被依赖
        datatype (str): 数据类型
        project (str): 项目        
    '''


    name = TagsField(name='name',tag_key=True)
    version = TagsField(name='version',tag_key=True)
    summary = TagsField(name='summary',tag_key=False)
    config = TagsField(name='config',tag_key=False)
    remark = TagsField(name='remark',tag_key=False)
    requiredby = TagsField(name='requiredby',tag_key=False)
    datatype = TagsField(name='datatype',tag_key=False)
    project = TagsField(name='project',tag_key=False)    

    

###################################################################################################################
###################################################################################################################





from fiche.cons import * 


### fiche存储
### fiche存储--observer_info
fiche_observer_info_storage = MongoManager(host='192.168.1.18',
                                           port=27017,
                                           username='admin',
                                           password='123456',
                                           database='fiche',
                                           collection='observer_info')
### fiche存储--algorithm_info
fiche_algorithm_info_storage = MongoManager(host='192.168.1.18',
                                            port=27017,
                                            username='admin',
                                            password='123456',
                                            database='fiche',
                                            collection='algorithm_info')
### fiche存储--model_info
fiche_model_info_storage = MongoManager(host='192.168.1.18',
                                        port=27017,
                                        username='admin',
                                        password='123456',
                                        database='fiche',
                                        collection='model_info')                                            
### fiche存储--parameter_info
fiche_parameter_info_storage = MongoManager(host='192.168.1.18',
                                            port=27017,
                                            username='admin',
                                            password='123456',
                                            database='fiche',
                                            collection='parameter_info')
### fiche存储--application_info
fiche_application_info_storage = MongoManager(host='192.168.1.18',
                                              port=27017,
                                              username='admin',
                                              password='123456',
                                              database='fiche',
                                              collection='application_info')
### fiche存储--dataset_info
fiche_dataset_info_storage = MongoManager(host='192.168.1.18',
                                          port=27017,
                                          username='admin',
                                          password='123456',
                                          database='fiche',
                                          collection='dataset_info')                                          


### 观察者信息
test_observer_info = Observer_Info(name = 'test',
                                   version = '001',
                                   summary = 'test oberver',
                                   config = 'test observer config',
                                   remark = 'test remark')
test_observer_info.save(storage=fiche_observer_info_storage)
### 算法包信息
test_algorithm_info = Algorithm_Info(name = 'test',
                                     version = '002',
                                     summary = 'this is a test algorithm package',
                                     config = 'bash xxxxx.sh',
                                     remark = 'test algorithm remark',
                                     homepage = 'xxx.xxx.xxx.xxx/xxxx',
                                     author = 'shihua',
                                     authoremail = '15021408795@163.com',
                                     license = 'MIT',
                                     requires = 'numpy,sklearn,tensorflow',
                                     requiredby = 'WPFsystem')
test_algorithm_info.save(storage=fiche_algorithm_info_storage)


### 模型信息
test_model_info = Model_Info(name = 'test',
                             version = '001',
                             summary = 'this is a test model',
                             config = 'bash xxxxxxx.sh',
                             remark = 'test model remark',
                             requires = 'test',
                             data = 'test data set',
                             algorithm = 'test algorithm')
test_model_info.save(storage=fiche_model_info_storage)   


### 参数信息
test_parameter_info = Parameter_Info(name = 'test',
                                     version = '001',
                                     summary = 'this is a test parameter',
                                     config = 'bash xxxxxxxxxxx.sh',
                                     remark = 'test parameter remark',
                                     datatype = 'test datatype',
                                     requiredby = 'test model')
test_parameter_info.save(storage=fiche_parameter_info_storage)


### 应用信息
test_application_info = Application_Info(name = 'test',
                                         version = '001',
                                         summary = 'this is a test application',
                                         config = 'bash xxxxxxxxxxxxx.sh',
                                         remark = 'test application remark',
                                         requires = 'xxx algorithm,xxxxx data',
                                         project = 'wpf system')
test_application_info.save(storage=fiche_application_info_storage)                                         


### 数据集信息
test_dataset_info = Dataset_Info(name = 'test',
                                 version = '001',
                                 summary = 'this is a test dataset',
                                 config = 'bash xxxxxxxxxxxxx.sh',
                                 remark = 'test dataset',
                                 requiredby = 'test model,test application',
                                 datatype = 'test datatype',
                                 project = 'pv system')
test_dataset_info.save(storage=fiche_dataset_info_storage)





# -*- coding: utf-8 -*-
# author:shihua
# coder:shihua
# 这是一个观察者模式类，主要采用推模型的功能设计。
"""
模块介绍
-------

这是一个观察者模式类，主要采用推模型的功能设计。

设计模式：

    （1）  无 

关键点：    

    （1）观察者模式

主要功能：            

    （1）主从节点信息管理                                 

使用示例
-------


类说明
------

"""



####### 载入程序包 ##########################################################
############################################################################



from abc import ABCMeta,abstractclassmethod
from fiche.cons import *



####### 观察者模式类 ########################################################
### 设计模式：                                                           ###
### （1）无                                                              ###
### 关键点：                                                             ###
### （1）观察者模式                                                      ###
### 主要功能：                                                           ###
### （1）主从节点信息管理                                                 ###
############################################################################


####### 观察者类 ###############################################################
###############################################################################



class Observer(metaclass = ABCMeta):
    '''
    类介绍：

        这是一个观察者类，主要功能用于接受master节点发送的通知并处理，主要技术抽象方法
    '''

    @abstractclassmethod
    def update(self):
        '''
        方法功能：

            定义一个更新观察者的抽象方法
        '''

        pass



####### 被观察者类 #################################################################
###################################################################################



class Observable(object):
    '''
    类介绍：

        这是一个被观察者类，主要功能管理观察者、向观察者下发通知、与元数据信息管理通信。主要技术类继承。
    '''


    @staticmethod
    def add_observer(storage,
                     observer_name,
                     observer_version,
                     observer_summary='default summary',
                     observer_config='default config',
                     observer_remark='default remark'):
        '''
        方法功能：

            定义一个添加观察者的方法

        参数：
            storage (object): 存储对象MongoDB
            observer_name (str): 观察者名称
            observer_version (str): 观察者版本
            observer_summary (str): 观察者简述
            obserser_config (str): 观察者配置
            observer_remark (str): 观察者备注

        返回：
            result (str): 结果信息
        '''

        ### 整理观察者信息
        tmp_observer_info = Observer_Info(name = observer_name,
                                          version = observer_version,
                                          summary = observer_summary,
                                          config = observer_config,
                                          remark = observer_remark)
        ### 向主节点保存观察者信息                                          
        tmp_observer_info.save(storage=storage)
        ### 执行结果信息
        result = '{} add well done!'.format(observer_name)
        print(result)

        return result


    @staticmethod
    def remove_observer(storage,observer_name,observer_version):
        '''
        方法功能：

            定义一个删除观察者的方法

        参数：
            storage (object): 存储对象MongoDB 
            observer_name (str): 观察者名称
            observer_version (str): 观察者版本

        返回：
            result (str): 结果信息
        '''

        ### 配置要删除的观察者参数
        tmp_delete_query = {'name':observer_name,'version':observer_version}
        ### 删除指定观察者
        storage.delete_info(tmp_delete_query)
        result = '{}--{}remove well done!'.format(observer_name,observer_version)
        print(result)

        return result


    @staticmethod
    def get_observers(storage):
        '''
        方法功能：

            定义一个获取观察者的方法

        参数：
            storage (object): 存储对象MongoDB

        返回：
            observers_list (list): 观察者列表
        '''

        ### 获取mongodb观察者迭代对象
        observers_iter = storage.client.find()
        ### 整理观察者列表，包括去重操作
        observers_list = []         
        for i in observers_iter:
            observers_list.append((i['name'],i['version']))
        observers_list = list(set(observers_list))

        return observers_list


    @staticmethod
    def update_observer(*args,**kwargs):
        '''
        方法功能：

            定义一个更新观察者的方法，暂时不提供。

        参数：
            observer_name (str): 观察者名称
            oberver_version (str): 观察者版本

        返回：
            result (str): 结果信息
        '''

        pass


    @staticmethod
    def notify_observer(*args,**kwargs):
        '''
        方法功能：

            定义一个通知观察者的方法，暂时不提供。

        参数：
            observer_name (str): 观察者名称

        返回：
            result (str): 结果信息
        '''

        pass



#################################################################################################
#################################################################################################





# -*- coding: utf-8 -*-
# author:shihua
# coder:shihua
# 这是一个fiche对外接口类，主要功能整理各种信息处理操作
"""
模块介绍
-------

这是一个fiche对外接口类，主要功能整理各种信息处理操作

设计模式：

    （1）  无 

关键点：    

    （1）信息处理

主要功能：            

    （1）整理各种信息处理操作                        

使用示例
-------


类说明
------

"""



####### 载入程序包 ##########################################################
############################################################################



from fiche.observer_mode import *
from fiche.cons import *



####### 观察者模式类 ########################################################
### 设计模式：                                                           ###
### （1）无                                                              ###
### 关键点：                                                             ###
### （1）信息处理                                                        ###
### 主要功能：                                                           ###
### （1）整理各种信息处理操作                                              ###
############################################################################


####### 信息处理操作 ###############################################################
###################################################################################



class FicheData(object):
    '''
    类介绍：

        这是一个fiche整理各种信息处理操作的接口类
    '''


    def __init__(self,host,username,password,port=27017,database='fiche'):### 默认参数放到最后
        '''
        属性功能：

            定义一个初始化属性功能的方法

        参数：
            host (str): mongodb的host
            port (int): mongodb的port，默认27017
            username (str): mongodb的用户名
            password (str): mongodb的密码
            database (str): mongodb中fiche对应的数据库
        返回：
            fichedata (object): fichedata实例
        '''

        self.host = host
        self.port = port
        self.username = username
        self.password = password
        self.database = database


    def add_observer(self,                     
                     observer_name,
                     observer_version,
                     observer_summary='default summary',
                     observer_config='default config',
                     observer_remark='default remark'):
        '''
        方法功能：

            定义一个添加观察者的方法

        参数：
            observer_name (str): 观察者名称
            observer_version (str): 观察者版本
            observer_summary (str): 观察者简述
            obserser_config (str): 观察者配置
            observer_remark (str): 观察者备注

        返回：
            result (str): 结果信息
        '''

        ### fiche存储--observer_info
        fiche_observer_info_storage = MongoManager(host=self.host,
                                                   port=self.port,
                                                   username=self.username,
                                                   password=self.password,
                                                   database=self.database,
                                                   collection='observer_info')                                   
        ### 添加观察者
        Observable.add_observer(storage=fiche_observer_info_storage,
                                observer_name=observer_name,
                                observer_version=observer_version,
                                observer_summary=observer_summary,
                                observer_config=observer_config,
                                observer_remark=observer_remark)     


    def remove_observer(self,observer_name,observer_version):
        '''
        方法功能：

            定义一个删除观察者的方法

        参数：
            observer_name (str): 观察者名称
            observer_version (str): 观察者版本

        返回：
            result (str): 结果信息
        '''                  

        ### fiche存储--observer_info
        fiche_observer_info_storage = MongoManager(host=self.host,
                                                   port=self.port,
                                                   username=self.username,
                                                   password=self.password,
                                                   database=self.database,
                                                   collection='observer_info')
        ### 删除观察者
        Observable.remove_observer(storage=fiche_observer_info_storage,
                                   observer_name=observer_name,
                                   observer_version=observer_version)    


    def show_obervers(self):
        '''
        方法功能：

            定义一个获取观察者的方法

        参数：
            无

        返回：
            observers_list (list): 观察者列表
        '''

        ### fiche存储--observer_info
        fiche_observer_info_storage = MongoManager(host=self.host,
                                                   port=self.port,
                                                   username=self.username,
                                                   password=self.password,
                                                   database=self.database,
                                                   collection='observer_info')
        ### 展示观察者
        observers_list = Observable.get_observers(storage=fiche_observer_info_storage)

        return observers_list      


    def query_observer_info(self,observer_name,observer_version):
        '''
        方法功能：

            定义一个查询观察者信息的方法

        参数：
            observer_name (str): 观察者名称
            observer_version (str): 观察者版本

        返回：
            observer_info_query (dict): 查询信息字典
        '''                                         

        ### fiche存储--observer_info
        fiche_observer_info_storage = MongoManager(host=self.host,
                                                   port=self.port,
                                                   username=self.username,
                                                   password=self.password,
                                                   database=self.database,
                                                   collection='observer_info')
        ### 查询observer_info
        observer_info_query = fiche_observer_info_storage.query_info({'name':observer_name,'version':observer_version})                                                                
        
        return observer_info_query


    def save_algorithm_info(self,
                            name,
                            version,
                            summary,
                            config,
                            remark,
                            homepage,
                            author,
                            authoremail,
                            license,
                            requires,
                            requiredby):
        '''
        方法功能：

            定义一个保存算法包信息方法

        参数：
            name (str): 算法包名称
            version (str): 版本
            summary (str): 简述
            config (str): 配置指导
            ramark (str): 备注
            homepage (str): 官方文档
            author (str): 作者
            authoremail (str): 作者邮箱
            license (str): 开源协议
            requires (str): 依赖
            requiredby (str): 被依赖
        
        返回：
            无
        '''                    

        ### fiche存储--algorithm_info
        fiche_algorithm_info_storage = MongoManager(host=self.host,
                                                    port=self.port,
                                                    username=self.username,
                                                    password=self.password,
                                                    database=self.database,
                                                    collection='algorithm_info')                
        ### 算法包信息
        tmp_algorithm_info = Algorithm_Info(name = name,
                                            version = version,
                                            summary = summary,
                                            config = config,
                                            remark = remark,
                                            homepage = homepage,
                                            author = author,
                                            authoremail = authoremail,
                                            license = license,
                                            requires = requires,
                                            requiredby = requiredby)
        tmp_algorithm_info.save(storage=fiche_algorithm_info_storage)   


    def query_algorithm_info(self,algorithm_name,algorithm_version):
        '''
        方法功能：

            定义一个查询算法信息的方法

        参数：
            algorithm_name (str): 算法名称
            algorithm_version (str): 算法版本

        返回：
            algorithm_info_query (dict): 查询信息字典
        '''                                         

        ### fiche存储--algorithm_info
        fiche_algorithm_info_storage = MongoManager(host=self.host,
                                                    port=self.port,
                                                    username=self.username,
                                                    password=self.password,
                                                    database=self.database,
                                                    collection='algorithm_info')
        ### 查询algorithm_info
        algorithm_info_query = fiche_algorithm_info_storage.query_info({'name':algorithm_name,'version':algorithm_version})                                                                
        
        return algorithm_info_query


    def save_model_info(self,
                        name,
                        version,
                        summary,
                        config,
                        remark,
                        requires,
                        data,
                        algorithm):
        '''
        方法功能：

            定义一个保存模型信息的方法

        参数:
            name (str): 算法包名称
            version (str): 版本
            summary (str): 简述
            data (str): 依赖数据
            algorithm (str): 依赖算法
            requires (str): 依赖
            config (str): 配置
            remark (str): 备注    

        返回：
            无        
        '''

        ### fiche存储--model_info
        fiche_model_info_storage = MongoManager(host=self.host,
                                                port=self.port,
                                                username=self.username,
                                                password=self.password,
                                                database=self.database,
                                                collection='model_info')
        ### 模型信息
        tmp_model_info = Model_Info(name = name,
                                    version = version,
                                    summary = summary,
                                    config = config,
                                    remark = remark,
                                    requires = requires,
                                    data = data,
                                    algorithm = algorithm)
        tmp_model_info.save(storage=fiche_model_info_storage)                                            


    def query_model_info(self,model_name,model_version):
        '''
        方法功能：

            定义一个查询模型信息的方法

        参数：
            model_name (str): 模型名称
            model_version (str): 模型版本

        返回：
            model_info_query (dict): 查询信息字典
        '''                                         

        ### fiche存储--model_info
        fiche_model_info_storage = MongoManager(host=self.host,
                                                port=self.port,
                                                username=self.username,
                                                password=self.password,
                                                database=self.database,
                                                collection='model_info')
        ### 查询model_info
        model_info_query = fiche_model_info_storage.query_info({'name':model_name,'version':model_version})                                                                
        
        return model_info_query


    def save_parameter_info(self,
                            name,
                            version,
                            summary,
                            config,
                            remark,
                            datatype,
                            requiredby):
        '''
        方法功能：

            定义一个保存参数信息的方法
        
        参数：
            name (str): 算法包名称
            version (str): 版本
            summary (str): 简述
            config (str): 配置指导
            remark (str): 备注
            datatype (str): 类型
            requiredby (str): 被依赖 

        返回：
            无
        '''                            

        ### fiche存储--parameter_info
        fiche_parameter_info_storage = MongoManager(host=self.host,
                                                    port=self.port,
                                                    username=self.username,
                                                    password=self.password,
                                                    database=self.database,
                                                    collection='parameter_info')
        ### 参数信息
        tmp_parameter_info = Parameter_Info(name = name,
                                            version = version,
                                            summary = summary,
                                            config = config,
                                            remark = remark,
                                            datatype = datatype,
                                            requiredby = requiredby)
        tmp_parameter_info.save(storage=fiche_parameter_info_storage)                                                    


    def query_parameter_info(self,parameter_name,parameter_version):
        '''
        方法功能：

            定义一个查询参数信息的方法

        参数：
            parameter_name (str): 参数名称
            parameter_version (str): 参数版本

        返回：
            parameter_info_query (dict): 查询信息字典
        '''                                         

        ### fiche存储--parameter_info
        fiche_parameter_info_storage = MongoManager(host=self.host,
                                                    port=self.port,
                                                    username=self.username,
                                                    password=self.password,
                                                    database=self.database,
                                                    collection='parameter_info')
        ### 查询parameter_info
        parameter_info_query = fiche_parameter_info_storage.query_info({'name':parameter_name,'version':parameter_version})                                                                
        
        return parameter_info_query


    def save_application_info(self,
                              name,
                              version,
                              summary,
                              config,
                              remark,
                              requires,
                              project):
        '''
        方法功能：

            定义一个保存应用信息的方法

        参数：
            name (str): 算法包名称
            version (str): 版本
            summary (str): 简述
            config (str): 配置指导
            ramark (str): 备注
            requires (str): 依赖
            project (str): 项目              
        
        返回：
            无
        '''        

        ### fiche存储--application_info
        fiche_application_info_storage = MongoManager(host=self.host,
                                                      port=self.port,
                                                      username=self.username,
                                                      password=self.password,
                                                      database=self.database,
                                                      collection='application_info')
        ### 应用信息
        tmp_application_info = Application_Info(name = name,
                                                version = version,
                                                summary = summary,
                                                config = config,
                                                remark = remark,
                                                requires = requires,
                                                project = project)
        tmp_application_info.save(storage=fiche_application_info_storage)                                                     


    def query_application_info(self,application_name,application_version):
        '''
        方法功能：

            定义一个查询应用信息的方法

        参数：
            application_name (str): 应用名称
            application_version (str): 应用版本

        返回：
            application_info_query (dict): 查询信息字典
        '''                                         

        ### fiche存储--application_info
        fiche_application_info_storage = MongoManager(host=self.host,
                                                    port=self.port,
                                                    username=self.username,
                                                    password=self.password,
                                                    database=self.database,
                                                    collection='application_info')
        ### 查询application_info
        application_info_query = fiche_application_info_storage.query_info({'name':application_name,'version':application_version})                                                                
        
        return application_info_query


    def save_dataset_info(self,
                          name,
                          version,
                          summary,
                          config,
                          remark,
                          requiredby,
                          datatype,
                          project):
        '''
        方法功能：

            定义一个保存数据集信息的方法

        参数：
            name (str): 算法包名称
            version (str): 版本
            summary (str): 简述
            config (str): 配置指导
            remark (str): 备注
            requiredby (str): 被依赖
            datatype (str): 数据类型
            project (str): 项目 

        返回：
            无
        '''   

        ### fiche存储--dataset_info
        fiche_dataset_info_storage = MongoManager(host=self.host,
                                                  port=self.port,
                                                  username=self.username,
                                                  password=self.password,
                                                  database=self.database,
                                                  collection='dataset_info')     
        ### 数据集信息
        tmp_dataset_info = Dataset_Info(name = name,
                                        version = version,
                                        summary = summary,
                                        config = config,
                                        remark = remark,
                                        requiredby = requiredby,
                                        datatype = datatype,
                                        project = project)
        tmp_dataset_info.save(storage=fiche_dataset_info_storage)                                                


    def query_dataset_info(self,dataset_name,dataset_version):
        '''
        方法功能：

            定义一个查询应用信息的方法

        参数：
            dataset_name (str): 应用名称
            dataset_version (str): 应用版本

        返回：
            dataset_info_query (dict): 查询信息字典
        '''                                         

        ### fiche存储--application_info
        fiche_dataset_info_storage = MongoManager(host=self.host,
                                                    port=self.port,
                                                    username=self.username,
                                                    password=self.password,
                                                    database=self.database,
                                                    collection='dataset_info')
        ### 查询dataset_info
        dataset_info_query = fiche_dataset_info_storage.query_info({'name':dataset_name,'version':dataset_version})                                                                
        
        return dataset_info_query       



############################################################################################################################################
############################################################################################################################################





# -*- coding: utf-8 -*-
# author:shihua
# coder:shihua
# 这是一个MongoDB专属的ORM模型基础模块
"""
模块介绍
-------

这是一个MongoDB专属的ORM模型基础模块

设计模式：

    （1）  无 

关键点：    

    （1）type与__new__技术

主要功能：            

    （1）MongoDB专属ORM模型                                   

使用示例
-------


类说明
------

"""



####### 载入程序包 ##########################################################
############################################################################



from fiche.util import *
import datetime



####### InfluxDBORM模型类 ##################################################
### 设计模式：                                                           ###
### （1）无                                                              ###
### 关键点：                                                             ###
### （1）type与__new__技术                                               ###
### 主要功能：                                                           ###
### （1）MongoDB专属ORM模型                                              ###
############################################################################


####### 数据类型类 #############################################################
###############################################################################



class Field(object):
    """
    类介绍：

        这是一个数据模型中的表达数据变量类型的基础类
    """


    def __init__(self,name,field_type,tag_key,default):
        """
        属性功能：

            定义一个初始化属性的方法

        参数：
            name (str): 数据类型名称
            field_type (str): 数据类型
            tag_key （bool): 是否是标签数据类型
            default (obj): 默认值
        """

        self.name = name
        self.field_type = field_type
        self.tag_key = tag_key
        self.default = default



class FieldsField(Field):
    """
    类介绍：

        这是一个数据模型中的表达fields数据变量类型的基础类
    """    


    def __init__(self,name,field_type = 'all',tag_key = False,default = None):
        """
        属性功能：

            定义一个初始化属性的方法

        参数：
            name (str): 数据类型名称
            field_type (str): 数据类型all
            tag_key （bool): 是否是标签数据类型False
            default (obj): 默认值
        """

        super().__init__(name,field_type,tag_key,default)




class TagsField(Field):
    """
    类介绍：

        这是一个数据模型中的表达fields数据变量类型的基础类
    """    


    def __init__(self,name,field_type = 'str',tag_key = True,default = None):
        """
        属性功能：

            定义一个初始化属性的方法

        参数：
            name (str): 数据类型名称
            field_type (str): 数据类型str
            tag_key （bool): 是否是标签数据类型True
            default (obj): 默认值
        """        

        super().__init__(name,field_type,tag_key,default)



####### 数据模型元类 #########################################################
#############################################################################



class ModelMetaClass(type):
    """
    类介绍：

        这是一个MongoDB数据模型构建类，主要用于抽象数据类型的构建过程。
    """


    def __new__(cls,name,obj,attr):
        """
        属性功能：

            定义一个MongoDB数据模型构建的魔法方法

        参数：
            cls (object): 构建类自身
            name (str): 构建类名称
            obj (object): 构建类父类
            attr (object): 构建类属性，包括方法

        返回：
            obj (object): 构建类
        """

        if name == 'Models':
            return type.__new__(cls,name,obj,attr)
        mappings = {}
        for k,v in attr.items():
            if isinstance(v,Field):
                mappings[k] = v
        for k in mappings.keys():
            attr.pop(k)
        attr['__table_name__'] = name
        attr['__mappings__'] = mappings
        return type.__new__(cls,name,obj,attr)



####### 数据模型基础类 #####################################################
###########################################################################



class Models(dict,metaclass = ModelMetaClass):
    """
    类介绍：

        这是一个InfluxDB数据模型类，主要用于抽象InfluxDB数据表结果。
    """


    def __init__(self,**kwargs):
        """
        属性功能：

            定义一个初始化的方法
        """

        super().__init__(**kwargs)


    def __getattr__(self,key):
        """
        方法功能：

            定义一个获取属性并检查的魔法方法

        参数：
            key (str): 属性关键字

        返回：
            属性对象
        """

        try:
            return self[key]
        except KeyError:
            raise AttributeError("No attribute '{}'".format(key))


    def __setattr__(self,key,value):
        """
        方法功能：

            定义一个设置属性的魔法方法

        参数：
            key (str): 属性关键字
            value (Object): 数据值对象

        返回：
            无
        """

        self[key] = value


    def save(self,storage):
        '''
        方法功能：

            定义一个保存数据模型中的数据到mongodb中的方法

        参数：
            storage (object): 存储器对象(MongoDB)

        返回：
            result (str): 执行结果信息
        '''

        ### 解析数据模型中的数据为字典
        tmp_data_dict = {}
        for tmp_key in list(self.__mappings__.keys()):
            tmp_data_dict[tmp_key] = self[tmp_key]
            tmp_data_dict['time'] = datetime.datetime.now()
        tmp_insert_list = [tmp_data_dict]
        ### 将解析后的字典加载进MongoDB
        storage.insert_info(tmp_insert_list)
        result = "MongoDB save well done!"
        print(result)

        return result


##########################################################################################
##########################################################################################





# -*- coding: utf-8 -*-
# author:shihua
# coder:shihua
# 这是一个fiche服务应用路由类，主要使用fastspi搭建ASGI服务器
"""
模块介绍
-------

这是一个fiche服务应用路由类，主要使用fastspi搭建ASGI服务器

设计模式：

    无

关键点：    

    （1）fastapi

主要功能：            

    （1）fiche服务应用路由
                                                     
使用示例
-------


类说明
------

"""



####### 载入程序包 ##########################################################
############################################################################



from fiche.data import *
from fiche.cons import fiche_config_yaml
from fastapi import FastAPI
import uvicorn
from fastapi import APIRouter



####### fiche服务应用路由 ###################################################
### 设计模式：                                                            ###
###     无                                                               ###
### 关键点：                                                              ###
### （1）fastapi                                                          ###
### 主要功能：                                                            ###
### （1）fiche服务应用路由                                                 ###
############################################################################



####### fiche服务观察者路由 ###########################################################################
######################################################################################################



observer_router = APIRouter()


### 增加观察者路由
@observer_router.get("/add_observer")
async def add_observer(observer_name,observer_version):
    '''
    函数功能：

        定义一个添加观察者信息的函数

    参数：
        observer_name (str): 观察者名称
        observer_version (str): 观察者版本

    返回：
        result (json): json结果字符串
    '''

    ### 从fiche的yaml配置文件中加载参数，并获得fiche的数据接口实例
    fiche = FicheData(host=fiche_config_yaml['host'],
                      port=fiche_config_yaml['port'],
                      username=fiche_config_yaml['username'],
                      password=fiche_config_yaml['password'],
                      database=fiche_config_yaml['database'])
    ### 执行增加观察者操作                      
    fiche.add_observer(observer_name=observer_name,observer_version=observer_version)
    result = {'info':'Add observer well done!'}

    return result


### 查询观察者
@observer_router.get("/query_observer")
async def query_observer(observer_name,observer_version):
    '''
    函数功能：

        定义一个查询观察者信息的函数

    参数：
        observer_name (str): 观察者名称
        observer_version (str): 观察者版本

    返回：
        result (json): json结果字符串
    '''

    ### 从fiche的yaml配置文件中加载参数，并获得fiche的数据接口实例
    fiche = FicheData(host=fiche_config_yaml['host'],
                      port=fiche_config_yaml['port'],
                      username=fiche_config_yaml['username'],
                      password=fiche_config_yaml['password'],
                      database=fiche_config_yaml['database'])
    ### 执行查询观察者操作                      
    tmp_observer_info_query = fiche.query_observer_info(observer_name=observer_name,observer_version=observer_version)
    result = {'data':tmp_observer_info_query}

    return result


### 删除观察者    
@observer_router.get("/remove_observer")
async def remove_observer(observer_name,observer_version):
    '''
    函数功能：

        定义一个删除观察者信息的函数

    参数：
        observer_name (str): 观察者名称
        observer_version (str): 观察者版本

    返回：
        result (json): json结果字符串
    '''

    ### 从fiche的yaml配置文件中加载参数，并获得fiche的数据接口实例
    fiche = FicheData(host=fiche_config_yaml['host'],
                      port=fiche_config_yaml['port'],
                      username=fiche_config_yaml['username'],
                      password=fiche_config_yaml['password'],
                      database=fiche_config_yaml['database'])
    ### 执行删除观察者操作
    fiche.remove_observer(observer_name=observer_name,observer_version=observer_version)
    result = {'info':'{}--{} Remove observer well done!'.format(observer_name,observer_version)}

    return result                          



####### fiche服务各个信息管理路由 ###########################################################################
######################################################################################################


info_manager_router = APIRouter()


### 保存算法信息
@info_manager_router.get("/save_algorithm_info")
async def save_algorithm_info(name,version,summary,config,remark,homepage,author,authoremail,license,requires,requiredby):
    '''
    函数功能：

        定义一个保存算法信息的函数

    参数：
        name (str): 算法包名称
        version (str): 版本
        summary (str): 简述
        config (str): 配置指导
        ramark (str): 备注
        homepage (str): 官方文档
        author (str): 作者
        authoremail (str): 作者邮箱
        license (str): 开源协议
        requires (str): 依赖
        requiredby (str): 被依赖
    
    返回：
        result (json): json结果字符串
    '''

    ### 从fiche的yaml配置文件中加载参数，并获得fiche的数据接口实例
    fiche = FicheData(host=fiche_config_yaml['host'],
                      port=fiche_config_yaml['port'],
                      username=fiche_config_yaml['username'],
                      password=fiche_config_yaml['password'],
                      database=fiche_config_yaml['database'])
    ### 保存算法信息
    fiche.save_algorithm_info(name=name,
                              version=version,
                              summary=summary,
                              config=config,
                              remark=remark,
                              homepage=homepage,
                              author=author,
                              authoremail=authoremail,
                              license=license,
                              requires=requires,
                              requiredby=requiredby)
    result = {'info':'Save algorithm info well done!'}

    return result                      


### 查询算法信息
@info_manager_router.get("/query_algorithm_info")
async def query_algorithm_info(algorithm_name,algorithm_version):
    '''
    函数功能：

        定义一个查询算法信息的函数

    参数：
        algorithm_name(str): 算法名称
        algorithm_version(str): 算法版本

    返回：
        result (json): json结果字符串
    '''

    ### 从fiche的yaml配置文件中加载参数，并获得fiche的数据接口实例
    fiche = FicheData(host=fiche_config_yaml['host'],
                      port=fiche_config_yaml['port'],
                      username=fiche_config_yaml['username'],
                      password=fiche_config_yaml['password'],
                      database=fiche_config_yaml['database'])
    ### 查询算法信息
    tmp_algorithm_info_query = fiche.query_algorithm_info(algorithm_name=algorithm_name,algorithm_version=algorithm_version)           
    result = {'data':tmp_algorithm_info_query}

    return result


### 保存模型信息
@info_manager_router.get("/save_model_info")
async def save_model_info(name,version,summary,config,remark,requires,data,algorithm):
    '''
    函数功能：

        定义一个保存模型信息的函数

    参数：
        name (str): 算法包名称
        version (str): 版本
        summary (str): 简述
        data (str): 依赖数据
        algorithm (str): 依赖算法
        requires (str): 依赖
        config (str): 配置
        remark (str): 备注

    返回：
        result (json): json结果字符串
    '''

    ### 从fiche的yaml配置文件中加载参数，并获得fiche的数据接口实例
    fiche = FicheData(host=fiche_config_yaml['host'],
                      port=fiche_config_yaml['port'],
                      username=fiche_config_yaml['username'],
                      password=fiche_config_yaml['password'],
                      database=fiche_config_yaml['database'])
    ### 保存模型信息
    fiche.save_model_info(name=name,
                          version=version,
                          summary=summary,
                          config=config,
                          remark=remark,
                          requires=requires,
                          data=data,
                          algorithm=algorithm)
    result = {'info':'Save model info well done!'}

    return result


### 查询模型信息
@info_manager_router.get("/query_model_info")
async def query_model_info(model_name,model_version):
    '''
    函数功能：

        定义一个查询模型信息的函数

    参数：
        model_name (str): 模型名称
        model_version (str): 模型版本

    返回：
        result (json): json结果字符串
    '''

    ### 从fiche的yaml配置文件中加载参数，并获得fiche的数据接口实例
    fiche = FicheData(host=fiche_config_yaml['host'],
                      port=fiche_config_yaml['port'],
                      username=fiche_config_yaml['username'],
                      password=fiche_config_yaml['password'],
                      database=fiche_config_yaml['database'])   
    ### 查询模型信息
    tmp_model_info_query = fiche.query_model_info(model_name=model_name,model_version=model_version)
    result = {'data':tmp_model_info_query}

    return result                                                 


### 保存参数信息
@info_manager_router.get("/save_parameter_info")
async def save_parameter_info(name,version,summary,config,remark,datatype,requiredby):
    '''
    函数功能：

        定义一个保存参数信息的函数

    参数：
        name (str): 算法包名称
        version (str): 版本
        summary (str): 简述
        config (str): 配置指导
        remark (str): 备注
        datatype (str): 类型
        requiredby (str): 被依赖

    返回：
        result (json): json结果字符串
    '''

    ### 从fiche的yaml配置文件中加载参数，并获得fiche的数据接口实例
    fiche = FicheData(host=fiche_config_yaml['host'],
                      port=fiche_config_yaml['port'],
                      username=fiche_config_yaml['username'],
                      password=fiche_config_yaml['password'],
                      database=fiche_config_yaml['database']) 
    ### 保存参数信息
    fiche.save_parameter_info(name=name,
                              version=version,
                              summary=summary,
                              config=config,
                              remark=remark,
                              datatype=datatype,
                              requiredby=requiredby)
    result = {'info':'Save parameter info well done!'}

    return result             


### 查询参数信息
@info_manager_router.get("/query_parameter_info")
async def query_parameter_info(parameter_name,parameter_version):
    '''
    函数功能：

        定义一个查询参数信息的函数

    参数：
        parameter_name (str): 参数名称
        parameter_version (str): 参数版本

    返回：
        result (json): json结果字符串
    '''

    ### 从fiche的yaml配置文件中加载参数，并获得fiche的数据接口实例
    fiche = FicheData(host=fiche_config_yaml['host'],
                      port=fiche_config_yaml['port'],
                      username=fiche_config_yaml['username'],
                      password=fiche_config_yaml['password'],
                      database=fiche_config_yaml['database'])
    ### 查询参数信息
    tmp_parameter_info_query = fiche.query_parameter_info(parameter_name=parameter_name,parameter_version=parameter_version)                                           
    result = {'data':tmp_parameter_info_query}

    return result


### 保存应用信息
@info_manager_router.get("/save_application_info")
async def save_application_info(name,version,summary,config,remark,requires,project):
    '''
    函数功能：

        定义一个保存应用信息的函数

    参数：
        name (str): 算法包名称
        version (str): 版本
        summary (str): 简述
        config (str): 配置指导
        remark (str): 备注
        requires (str): 依赖
        project (str): 项目 

    返回：
        result (json): json结果字符串
    '''

    ### 从fiche的yaml配置文件中加载参数，并获得fiche的数据接口实例
    fiche = FicheData(host=fiche_config_yaml['host'],
                      port=fiche_config_yaml['port'],
                      username=fiche_config_yaml['username'],
                      password=fiche_config_yaml['password'],
                      database=fiche_config_yaml['database']) 
    ### 保存应用信息
    fiche.save_application_info(name=name,
                                version=version,
                                summary=summary,
                                config=config,
                                remark=remark,
                                requires=requires,
                                project=project)
    result = {'info':'Save application info well done!'}

    return result


### 查询应用信息
@info_manager_router.get("/query_application_info")
async def query_application_info(application_name,application_version):
    '''
    函数功能：

        定义一个查询应用信息的函数

    参数：
        application_name (str): 应用名称
        application_version (str): 应用版本

    返回：
        result (json): json结果字符串
    '''

    ### 从fiche的yaml配置文件中加载参数，并获得fiche的数据接口实例
    fiche = FicheData(host=fiche_config_yaml['host'],
                      port=fiche_config_yaml['port'],
                      username=fiche_config_yaml['username'],
                      password=fiche_config_yaml['password'],
                      database=fiche_config_yaml['database']) 
    ### 查询应用信息
    tmp_application_info_query = fiche.query_application_info(application_name=application_name,application_version=application_version)
    result = {'data':tmp_application_info_query}

    return result


### 保存数据信息
@info_manager_router.get("/save_dataset_info")
async def save_dataset_info(name,version,summary,config,remark,requiredby,datatype,project):
    '''
    函数功能：

        定义一个保存数据信息的函数

    参数：
        name (str): 算法包名称
        version (str): 版本
        summary (str): 简述
        config (str): 配置指导
        remark (str): 备注
        requiredby (str): 被依赖
        datatype (str): 数据类型
        project (str): 项目 

    返回：
        result (json): json结果字符串
    '''

    ### 从fiche的yaml配置文件中加载参数，并获得fiche的数据接口实例
    fiche = FicheData(host=fiche_config_yaml['host'],
                      port=fiche_config_yaml['port'],
                      username=fiche_config_yaml['username'],
                      password=fiche_config_yaml['password'],
                      database=fiche_config_yaml['database']) 
    ### 保存数据信息
    fiche.save_dataset_info(name=name,
                            version=version,
                            summary=summary,
                            config=config,
                            remark=remark,
                            requiredby=requiredby,
                            datatype=datatype,
                            project=project)    
    result = {'info':'Save dataset info well done!'}

    return result


### 查询数据信息
@info_manager_router.get("/query_dataset_info")
async def query_dataset_info(dataset_name,dataset_version):
    '''
    函数功能：

        定义一个查询数据信息的函数

    参数：
        dataset_name (str): 数据名称
        dataset_version (str): 数据版本

    返回：
        result (json): json结果字符串
    '''

    ### 从fiche的yaml配置文件中加载参数，并获得fiche的数据接口实例
    fiche = FicheData(host=fiche_config_yaml['host'],
                      port=fiche_config_yaml['port'],
                      username=fiche_config_yaml['username'],
                      password=fiche_config_yaml['password'],
                      database=fiche_config_yaml['database']) 
    ### 查询数据信息
    tmp_dataset_info_query = fiche.query_dataset_info(dataset_name=dataset_name,dataset_version=dataset_version)
    result = {'data':tmp_dataset_info_query}

    return result



###### fiche服务应用主路由 #################################################################################################################################
###########################################################################################################################################################



### 创建fastapi应用实例
app = FastAPI()
app.include_router(observer_router,prefix="/observer",tags=['observer'])
app.include_router(info_manager_router,prefix="/info_manager",tags=['info_manager'])



if __name__ == '__main__':
    uvicorn.run(app='service:app',host='0.0.0.0',port=11811,reload=True,debug=True) ### app参数需要使用字符串格式的时候，才可以添加reload和debug参数



##########################################################################################################################################################
##########################################################################################################################################################


from fiche.observer_mode import *
from fiche.cons import *



### fiche存储
### fiche存储--observer_info
fiche_observer_info_storage = MongoManager(host='192.168.1.18',
                                           port=27017,
                                           username='admin',
                                           password='123456',
                                           database='fiche',
                                           collection='observer_info')


### 添加观察者
Observable.add_observer(storage=fiche_observer_info_storage,
                        observer_name='test003',
                        observer_version='v003')                                           


### 删除观察者
Observable.remove_observer(storage=fiche_observer_info_storage,
                           observer_name='test',
                           observer_version='003')


### 展示观察者
observers_list = Observable.get_observers(storage=fiche_observer_info_storage) 
print(observers_list)


### 查询配置信息
### 查询observer_info
observer_info_query = fiche_observer_info_storage.query_info({'name':'test'})
print(observer_info_query)



# -*- coding: utf-8 -*-
# author:shihua
# coder:shihua
# 这是一个工具辅助模块
"""
模块介绍
-------

这是一个工具辅助模块

设计模式：

    （1）  无 

关键点：    

    （1）MongoDB信息管理

主要功能：            

    （1）MongoDB管理JSON                                

使用示例
-------


类说明
------

"""



####### 载入程序包 ##########################################################
############################################################################



import pymongo



####### MongoDB信息管理模型 ##################################################
### 设计模式：                                                           ###
### （1）无                                                              ###
### 关键点：                                                             ###
### （1）MongoDB信息管理                                                 ###
### 主要功能：                                                           ###
### （1）MongoDB管理JSON                                                 ###
############################################################################


####### MongoDB信息管理模型 #############################################################################
########################################################################################################



class MongoManager(object):
    '''
    类介绍：

        这是一个MongoDB数据库连接管理类
    '''


    def __init__(self,host,port,username,password,database,collection):
        '''
        属性功能：

            定义一个汇总数据库基本信息的属性方法

        参数:
            host (str): mongodb的ip地址
            port (int): mongodb的端口
            username (str): mongodb的用户名
            password (str): mongodb的密码
            database (str): 数据库名称
            collection (str): 集合名称
            client (object): mongodb客户端对象

        返回：
            无
        '''

        self.host = host
        self.port = port 
        self.username = username
        self.password = password
        self.database = database
        self.collection = collection
        self.client = self.create_mongodb_client()


    def create_mongodb_client(self):
        '''
        方法功能：

            定义一个连接mongodb的方法，从MongoManager实例参数中获取方法

        参数：
            无

        返回：
            mongodb_collection (object): mongodb客户端对象-mongodb集合
        '''

        mongodb_client = pymongo.MongoClient(host=self.host,port=self.port,username=self.username,password=self.password)
        mongodb_database = mongodb_client[self.database]
        database_list = mongodb_client.list_database_names()
        if self.database not in database_list: ### 执行数据库操作前，需要判断数据库是否存在
            mongodb_database.create_collection(self.collection)
        mongodb_collection = mongodb_database[self.collection]
        print('=============================>>>>>> fiche info {} create well done!'.format(self.collection))

        return mongodb_collection


    def insert_info(self,info_dict_list):
        '''
        方法功能：

            定义一个插入信息json串的方法

        参数：
            info_dict_list (list): 信息字典列表 

        返回：
            result (str): 插入成功信息
        '''

        self.client.insert_many(info_dict_list)
        result = "=============>>>>>> fiche info insert well done!"
        print(result)

        return result


    def update_info(self,query,update):
        '''
        方法功能：

            定义一个更新信息json串的方法

        参数：
            query (dict): mongodb查询字典
            update (dict): mongodb更新字典 

        返回：
            result (str): 更新成功信息
        '''

        self.client.update_many(query,update) 
        result = "=============>>>>>> fiche info update well done!"
        print(result)

        return result


    def query_info(self,query,view_index = 0):
        '''
        方法功能：

            定义一个查询信息json串的方法

        参数：
            query (dict): mongodb查询字典 

        返回：
            result_iter (object): 查询结果
        '''

        result_iter = self.client.find(query,projection={'_id': False})### 使用保护机制引入{'_id':false})来去除id额外索引
        result = "=============>>>>>> fiche info query well done!"
        print(result)

        return result_iter[view_index]       


    def delete_info(self,query):
        '''
        方法功能：

            定义一个删除信息json串的方法

        参数：
            query (dict): mongodb查询字典 

        返回：
            result (str): 删除成功信息
        '''

        self.client.delete_many(query)
        result = "=============>>>>>> fiche info delete well done!"
        print(result)

        return result         



##############################################################################################
##############################################################################################





from fiche.data import *


fiche = FicheData(host='192.168.1.18',
                  port=27017,
                  username='admin',
                  password='123456',
                  database='fiche')


### 增加观察者
# fiche.add_observer(observer_name='test001',observer_version='v001')
# fiche.add_observer(observer_name='test002',observer_version='v002')
# fiche.add_observer(observer_name='test_remove',observer_version='test_remove001')

### 查询观察者
tmp_observer_info_query = fiche.query_observer_info(observer_name='test003',observer_version='v003')
print(tmp_observer_info_query)

### 获取观察者列表
tmp_observers_list = fiche.show_obervers()
print(tmp_observers_list) 

### 删除观察者
# fiche.remove_observer(observer_name='test_remove',observer_version='test_remove001')

### 保存算法信息
# fiche.save_algorithm_info(name = 'test001',
#                           version = 'v001',
#                           summary = 'this is a test algorithm package',
#                           config = 'bash xxxxx.sh',
#                           remark = 'test algorithm remark',
#                           homepage = 'xxx.xxx.xxx.xxx/xxxx',
#                           author = 'shihua',
#                           authoremail = '15021408795@163.com',
#                           license = 'MIT',
#                           requires = 'numpy,sklearn,tensorflow',
#                           requiredby = 'WPFsystem')

### 查询算法信息
tmp_algorithm_info_query = fiche.query_algorithm_info(algorithm_name='test001',algorithm_version='v001')
print(tmp_algorithm_info_query)

### 保存模型信息
# fiche.save_model_info(name = 'test001',
#                       version = 'v001',
#                       summary = 'this is a test model',
#                       config = 'bash xxxxxxx.sh',
#                       remark = 'test model remark',
#                       requires = 'test',
#                       data = 'test data set',
#                       algorithm = 'test algorithm')

### 查询模型信息
tmp_model_info_query = fiche.query_model_info(model_name='test001',model_version='v001')
print(tmp_model_info_query)

### 保存参数信息
# fiche.save_parameter_info(name = 'test001',
#                           version = 'v001',
#                           summary = 'this is a test parameter',
#                           config = 'bash xxxxxxxxxxx.sh',
#                           remark = 'test parameter remark',
#                           datatype = 'test datatype',
#                           requiredby = 'test model')

### 查询参数信息
tmp_parameter_info_query = fiche.query_parameter_info(parameter_name='test001',parameter_version='v001')
print(tmp_parameter_info_query)

### 保存应用信息
# fiche.save_application_info(name = 'test001',
#                             version = 'v001',
#                             summary = 'this is a test application',
#                             config = 'bash xxxxxxxxxxxxx.sh',
#                             remark = 'test application remark',
#                             requires = 'xxx algorithm,xxxxx data',
#                             project = 'wpf system')

### 查询应用信息
tmp_application_info_query = fiche.query_application_info(application_name='test001',application_version='v001')
print(tmp_application_info_query)

### 保存数据信息
# fiche.save_dataset_info(name = 'test001',
#                         version = 'v001',
#                         summary = 'this is a test dataset',
#                         config = 'bash xxxxxxxxxxxxx.sh',
#                         remark = 'test dataset',
#                         requiredby = 'test model,test application',
#                         datatype = 'test datatype',
#                         project = 'pv system')

### 查询数据信息
tmp_dataset_info_query = fiche.query_dataset_info(dataset_name='test001',dataset_version='v001')
print(tmp_dataset_info_query)


from setuptools import setup,find_packages

### 打开readme文件流，使用utf-8编码
with open("README.mdown","r",encoding="utf-8") as fh:
        long_description = fh.read()

setup(
        ### 包与作者信息
        name = 'shihua-fiche',
        version = '0.1',
        author = 'shihua',
        author_email = "15021408795@163.com",
        python_requires = ">=3.9.12",
        license = "MIT",

        ### 源码与依赖
        packages = find_packages(),
        include_package_data = True,
        description = 'Fiche is a metadata management tool with MongoDB about algorithm info,model info,parameter info,application info,dataset info and more;supporting http interface and python sdk.',
        # install_requires = ['click','fastapi','rich','pymongo'],

        ### 包接入点，命令行索引
        entry_points = {
            'console_scripts': [
                'fichectl = fiche.cli:fiche'
            ]
        },

        ### pypi配置
        long_description = long_description,
        long_description_content_type = "text/markdown",
        url = "https://github.com/redblue0216/Fiche",
        classsifiers = [
                "Programming Language :: Python :: 3.9",
                "License :: OSI Approved :: MIT License",
                "Topic :: Scientific/Engineering :: Artificial Intelligence"
        ]      
)


from fiche.orm import *
from fiche.util import *
import datetime


### 构建数据模型基础类
class people_info(Models):
    name = FieldsField(name = 'name',tag_key = False)
    age = TagsField(name = 'age',tag_key = True)
    address = TagsField(name = 'address',tag_key = True)
    version= TagsField(name = 'version',tag_key= True)



# ### 构建数据模型
# people_info = people_info(name = 'd',age = 45,address = 'chengdu')
# people_info.save()
# print(list(people_info.__mappings__.keys()))
# tmp_data_dict = {}
# for tmp_key in list(people_info.__mappings__.keys()):
#     tmp_data_dict[tmp_key] = people_info[tmp_key]
#     tmp_data_dict['time'] = datetime.datetime.now()
# print(tmp_data_dict)
# tmp_insert_list = [tmp_data_dict]

### 二级API
### fiche_test信息表
fiche_storage = MongoManager(host='192.168.0.111',
                          port=27017,
                          username='admin',
                          password='123456',
                          database='fiche',
                          collection='test')
# print(fiche_test.client)
# fiche_test.insert_info(tmp_insert_list)
### 构建数据模型
people_info = people_info(name = 'e',age = 59,address = 'chengdu',version = '002')
### 保存数据到MongoDB
people_info.save(fiche_storage)
### 查询数据
test_query = {'name':'e','version':'002'}
query_result = fiche_storage.query_info(query = test_query)
print(query_result)



### 一级API
# # ### 增加一条信息
# # test_insert_list = [{'name':'a','age':16,'address':'beijing'},
# #                     {'name':'b','age':30,'address':'shanghai'},
# #                     {'name':'c','age':20,'address':'nanjing'}]                    
# # fiche_test.insert_info(test_insert_list)
# ### 更新一条信息
# # test_update_query = {'name':'a'}
# # test_update_update = {'$set':{'address':'dalian'}}
# # fiche_test.update_info(test_update_query,test_update_update)
# # ### 删除一条信息
# # test_delete_query = {'name':'c'}
# # fiche_test.delete_info(test_delete_query)
# ### 查询一条信息
# test_query = {'name':'b'}
# query_result = fiche_test.query_info(test_query)
# print(query_result)
# for i in query_result:
#     print(i,type(i))

### 二级API
